#!/bin/bash

ECHO=/bin/echo
GREP=/bin/grep
BASENAME=/usr/bin/basename
DIRNAME=/usr/bin/dirname
WC=/usr/bin/wc
MKTEMP=/bin/mktemp
MAIL=/usr/bin/mail
RM=/bin/rm
CAT=/bin/cat
SED=/bin/sed
HEAD=/usr/bin/head
MAWK=/usr/bin/mawk
DATE=/bin/date
CHOWN=/bin/chown
CHMOD=/bin/chmod
STAT=/usr/bin/stat
HOSTNAME=/bin/hostname
OPENSSL=/usr/bin/openssl

exist_user_rpc()
{
    USER_RPC=$1
    FILE_T=$2
    NET_RPC=$3

    RET=1
    USER_LINES_COUNT=`${CAT} ${FILE_T} | ${GREP} -v "\[${NET_RPC}\]" | ${MAWK} -F ":" '{print \$1;}' | ${GREP} -w "${USER_RPC}" | ${WC} -l`
    if [ ${USER_LINES_COUNT} -gt 0 ]
    then
	RET=0
    else
	RET=1
    fi

    return ${RET}
}

MIN_PASSWD_LEN=14

NAME_CALLED=`${BASENAME} $0`

CONF_PREFIX="rpcauth="
HELP_MSG="USAGE: ${NAME_CALLED} user password [ network (main | testnet | regtest) default main [ DONT_UPDATE_IF_EXISTS (true | false) default false ] ]"
PASS_LEN_MSG="The password must have almost ${MIN_PASSWD_LEN} characters"

USER_IN=$1
PASS_IN=$2
NET_IN=$3
DONT_UPDATE_IF_EXISTS=$4

if [ -z "${USER_IN}" ]
then
    ${ECHO} "${HELP_MSG}"
    exit 1
fi

if [ -z "${PASS_IN}" ]
then
    ${ECHO} "${HELP_MSG}"
    exit 1
fi

PASS_IN_LEN=`${ECHO} -n "${PASS_IN}" | ${WC} -m`

if [ ${PASS_IN_LEN} -lt ${MIN_PASSWD_LEN} ]
then
    ${ECHO} "${PASS_LEN_MSG}"
    exit 1
fi

STATIC_DEFAULT_CONF=/usr/lib/garlicoin-init/default/garlicoin-init
if [ -f ${STATIC_DEFAULT_CONF} ]
then
    . ${STATIC_DEFAULT_CONF}
fi

CONFIG_FILE=${SERVER_CONF}

CONFIG_DIR=`${DIRNAME} ${CONFIG_FILE}`
PASSWD_FILE="${CONFIG_DIR}/passwd.main.conf"

SALT_CALC=`${OPENSSL} rand -hex 32 | ${OPENSSL} dgst -md5 -hex -r | ${MAWK} '{print $1}'`

if [ "${NET_IN}" = "testnet" ]
then
    PASSWD_FILE="${CONFIG_DIR}/passwd.testnet.conf"
    NET_IN="test"
elif [ "${NET_IN}" = "regtest" ]
then
    PASSWD_FILE="${CONFIG_DIR}/passwd.regtest.conf"
else
    NET_IN="main"
fi

USER_OUT_PREFIX="${CONF_PREFIX}${USER_IN}"
HMAC_CALC=`${ECHO} -n "${PASS_IN}" | ${OPENSSL} dgst -sha256 -hmac ${SALT_CALC} -hex -r | ${MAWK} '{print $1 ;}'`
USER_OUT="${USER_OUT_PREFIX}:${SALT_CALC}"'$'"${HMAC_CALC}"

if exist_user_rpc ${USER_OUT_PREFIX} ${PASSWD_FILE} ${NET_IN}
then
    if ! [ "${DONT_UPDATE_IF_EXISTS}" = "true" ]
    then
	${SED} -i '/^'${USER_OUT_PREFIX}:'/c\'${USER_OUT} ${PASSWD_FILE}
    fi
else
    ${ECHO} "${USER_OUT}" >> ${PASSWD_FILE}
fi



